<?php if (! $this->compact): ?>
<div class="controls">
    <?= $this->tabs ?>
    <?= $this->render('list/components/selectioninfo.phtml') ?>
    <?= $this->paginator ?>
    <div class="sort-controls-container">
        <?= $this->limiter ?>
        <?= $this->sortBox ?>
    </div>
    <?= $this->filterEditor ?>
</div>
<?php endif ?>
<div class="content">
<?php if (! $comments->hasResult()): ?>
    <p><?= $this->translate('No comments found matching the filter') ?></p>
</div>
<?php return; endif ?>
    <table data-base-target="_next"
           class="table-row-selectable common-table multiselect"
           data-icinga-multiselect-url="<?= $this->href('monitoring/comments/show') ?>"
           data-icinga-multiselect-related="<?= $this->href("monitoring/comments") ?>"
           data-icinga-multiselect-data="comment_id">
    <thead class="print-only">
        <tr>
            <th><?= $this->translate('Type') ?></th>
            <th><?= $this->translate('Comment') ?></th>
        </tr>
    </thead>
    <tbody>
    <?php foreach ($comments->peekAhead($this->compact) as $comment): ?>
        <tr href="<?=  $this->href('monitoring/comment/show', array('comment_id' => $comment->id)) ?>">
            <td class="icon-col">
                <?= $this->partial('partials/comment/comment-description.phtml', array('comment' => $comment)) ?>
            </td>
            <td>
                <?= $this->partial(
                    'partials/comment/comment-detail.phtml',
                    array(
                        'comment'        => $comment,
                        'delCommentForm' => isset($delCommentForm) ? $delCommentForm : null
                        // Form is unset if the current user lacks the respective permission
                )) ?>
            </td>
        </tr>
    <?php endforeach ?>
    </tbody>
    </table>
<?php if ($comments->hasMore()): ?>
    <div class="dont-print action-links">
        <?= $this->qlink(
            $this->translate('Show More'),
            $this->url()->without(array('showCompact', 'limit')),
            null,
            array(
                'class'               => 'action-link',
                'data-base-target'    => '_next'
            )
        ) ?>
    </div>
<?php endif ?>
</div>
