<?php
/* Icinga Web 2 | (c) 2014 Icinga Development Team | GPLv2+ */
namespace Icinga\Protocol\File;

use Icinga\Exception\IcingaException;

/**
 * Exception thrown if a file reader specific error occurs
 */
class FileReaderException extends IcingaException
{
}
