<div class="controls">
    <?= $this->tabs ?>
</div>

<div class="content">
    <ul>
    <?php foreach ($modules as $module): /** @var \Icinga\Application\Modules\Module $module */ ?>
        <li><?= $this->qlink(
            $module->getTitle(),
            'doc/module/toc',
            array('moduleName' => $module->getName()),
            array('title' => sprintf(
                $this->translate('Show the documentation\'s table of contents for the %s'),
                $module->getTitle()
            ))
        ) ?></li>
    <?php endforeach ?>
    </ul>
</div>
