<?php
use Icinga\Module\Monitoring\Object\Host;
use Icinga\Module\Monitoring\Object\Service;
?>
<table class="state-table common-table" data-base-target="_next">
    <tbody>
    <?php
    foreach ($this->downtimes as $i => $downtime):
        if ($i > 5) {
            break;
        }
        if ($downtime->objecttype === 'service') {
            $this->isService = true;
            $this->stateName =  Service::getStateText($downtime->service_state);
        } else {
            $this->isService = false;
            $this->stateName =  Host::getStateText($downtime->host_state);
        }
        $this->downtime = $downtime;
        $this->displayComment = false;
        ?>
        <tr>
            <?= $this->render('partials/downtime/downtime-header.phtml') ?>
        </tr>
    <?php endforeach ?>
    </tbody>
</table>
<?php if ($downtimes->count() > 5): ?>
<p>
    <?= $this->qlink(
        sprintf($this->translate('List all %d downtimes'), $downtimes->count()),
        $listAllLink,
        null,
        array(
            'data-base-target'  => '_next',
            'icon'              => 'down-open'
        )
    ) ?>
</p>
<?php endif ?>
