<div class="controls">
    <?= $tabs ?>
</div>
<div class="content">
<?php if (isset($changePasswordForm)): ?>
    <h1><?= $this->translate('Account') ?></h1>
    <?= $changePasswordForm ?>
<?php endif ?>
    <h1><?= $this->translate('Preferences') ?></h1>
    <?= $form ?>
</div>
