<div class="controls">
    <?= /** @var \Icinga\Web\Widget\Tabs $tabs */ $tabs ?>
</div>
<div class="content">
    <ul>
        <li><?= $this->qlink(
            'Icinga Web 2',
            'doc/icingaweb/toc',
            null,
            array('title' => $this->translate('Show the documentation\'s table of contents for Icinga Web 2'))
        ) ?></li>
        <li><?= $this->qlink(
            $this->translate('Module documentations'),
            'doc/module/',
            null,
            array('title' => $this->translate('List all modules for which documentation is available'))
        ) ?></li>
    </ul>
</div>
